/**
* Copyright (C) 2014 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.service;

import java.util.List;
import java.util.UUID;

import org.springframework.stereotype.Component;

import eu.etaxonomy.cdm.api.application.CdmApplicationRemoteController;
import eu.etaxonomy.cdm.api.application.CdmApplicationState;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.taxeditor.session.ICdmEntitySession;

/**
 * @author cmathew
 * @date 14 Oct 2014
 */
@Component
public class CachedCommonServiceImpl implements ICachedCommonService {

    private static boolean cacheEnabled = true;

    public static boolean isCacheEnabled() {
        return cacheEnabled;
    }

    public static void setCacheEnabled(boolean cacheEnabled) {
        CachedCommonServiceImpl.cacheEnabled = cacheEnabled;
    }

    @Override
    public CdmBase find(Class<? extends CdmBase> clazz, int id) {
        CdmBase cdmEntity = CdmApplicationState.getCurrentAppConfig().getCommonService().find(clazz, id, getPropertyPaths(clazz));
        if(cdmEntity == null) {
            throw new NullPointerException("CDM Entity of type " + clazz.getName() + " with id " + id  + " is null.");
        }
        return cdmEntity;
    }

    private List<String> getPropertyPaths(Object obj) {
        List<String> propertyPaths = null;
        ICdmEntitySession cdmEntitySession =
                ((CdmApplicationRemoteController)CdmApplicationState.getCurrentAppConfig()).getCdmEntitySessionManager().getActiveSession();
        if(cdmEntitySession != null) {
            propertyPaths = cdmEntitySession.getPropertyPaths(obj);
        }
        return propertyPaths;
    }

    @Override
    public Object initializeCollection(UUID ownerUuid, String fieldName) {
        return  CdmApplicationState.getCurrentAppConfig().getCommonService().initializeCollection(ownerUuid, fieldName, getPropertyPaths(fieldName));
    }

    @Override
    public boolean isEmpty(UUID ownerUuid, String fieldName) {
            return  CdmApplicationState.getCurrentAppConfig().getCommonService().isEmpty(ownerUuid, fieldName);
    }

    @Override
    public int size(UUID ownerUuid, String fieldName) {
        return  CdmApplicationState.getCurrentAppConfig().getCommonService().size(ownerUuid, fieldName);
    }

    @Override
    public boolean contains(UUID ownerUuid, String fieldName, Object element) {
        return  CdmApplicationState.getCurrentAppConfig().getCommonService().contains(ownerUuid, fieldName, element);
    }

    @Override
    public boolean containsKey(UUID ownerUuid, String fieldName, Object key) {
        return  CdmApplicationState.getCurrentAppConfig().getCommonService().containsKey(ownerUuid, fieldName, key);
    }

    @Override
    public boolean containsValue(UUID ownerUuid, String fieldName, Object element) {
        return  CdmApplicationState.getCurrentAppConfig().getCommonService().containsValue(ownerUuid, fieldName, element);
    }

}
